//--------------------------------------------------------------------------
// Twisting Cylinder in OpenGL by Andreas Gustafsson (Shadow/Noice) (C) 2000
//--------------------------------------------------------------------------
#include "stdafx.h"
#include "Texture.h"

#define NR_DIVS 16
#define NR_SEGS 64


//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Skeleton";
HWND hwnd;

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;

double rad[NR_SEGS],sinpos[NR_SEGS],xpos[NR_SEGS];

// Use the demonstration texture.
CTexture gTexExample;
CTexture gTexExample2;
int updateFPS=0;
double curFPS=0;

float random(float high)
{
    return((float)((double)rand()/32768)*high);
}

// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	int i;

	glViewport (0, 0, (GLsizei) g_w, (GLsizei) g_h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective(55.0, (GLfloat) g_w/(GLfloat) g_h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.0, 0.0, -3.0);
	glClearColor(0.0,0.0,0.0,0.0);
	glEnable(GL_DEPTH_TEST);

	gTexExample.LoadJPG( "o008.jpg" );
	gTexExample2.LoadJPG( "backgrnd.jpg" );
	glEnable( GL_TEXTURE_2D );

	for(i=0;i<NR_SEGS;i++)
	{
		sinpos[i]=0;
		rad[i]=1;
	}

	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

void Shutdown() // Called right after the window is destroyed.
{
}

void setVertex(int i, int j)
{
	float col;

	glTexCoord2f((float)(sinpos[i]+j*(3.0/(double)NR_DIVS)),(float)(i*(3.0/(double)NR_SEGS)));
	j=j%NR_DIVS;
	col=(float)(1.2*rad[i]*sin(j*PI*2.0/(double)NR_DIVS));
	glColor3f(col,col,col);
	glVertex3f((float)(xpos[i]+rad[i]*cos(j*PI*2.0/(double)NR_DIVS)),(float)(i*(5.5/(double)NR_SEGS)),(float)(rad[i]*sin(j*PI*2.0/(double)NR_DIVS)));
}

void drawCyl()
{
	int i,j;

	gTexExample.Use();
	glBegin(GL_QUADS);
	for(i=0;i<(NR_SEGS-1);i++)
	{
		for(j=0;j<NR_DIVS;j++)
		{
			setVertex(i,j);
			setVertex(i+1,j);
			setVertex(i+1,j+1);
			setVertex(i,j+1);
		}
	}
	glEnd();
}

// Draw all the scene related stuff.
void Render()
{
	int i;
	char s[100];

	curFPS+=1.f/fDeltaTime;
	updateFPS++;
	if(updateFPS==20)
	{
		sprintf(s,"Twisting Cylinder by Andreas Gustafsson (Shadow/Noice) (C) 2000   -=[ FPS:%.0f ]=-",curFPS/20.0f);
		SetWindowText(hwnd,s);
		updateFPS=0;
		curFPS=0;
	}


	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	glScalef(2.3f,1.55f,1.0f);
	gTexExample2.Use(); 
	glBegin(GL_QUADS);
	glColor3f(1.0f,0.8f,0.8f);
	glTexCoord2f(0,0);
	glVertex2d(-1,-1);
	glColor3f(0.8f,1.0f,0.8f);
	glTexCoord2f(2,0);
	glVertex2d(1,-1);
	glColor3f(0.8f,0.8f,1.0f);
	glTexCoord2f(2,1);
	glVertex2d(1,1);
	glColor3f(1.0f,0.8f,1.0f);
	glTexCoord2f(0,1);
	glVertex2d(-1,1);
	glEnd();
	glPopMatrix();
	glPushMatrix();
	glRotatef( fTime*20, 0, 0, 1.f );
	glTranslatef(0,-2.75,0);
	drawCyl();
	for(i=0;i<NR_SEGS;i++)
	{
		sinpos[i]=0.53*sin(fTime*0.78+i/12.0)+0.45*sin(fTime*-1.23+i/18.0);;
		rad[i]=0.8+0.3*sin(fTime+i/8.0)+0.15*sin(fTime*-0.8+i/3.0);
		xpos[i]=0.25*sin(fTime*1.23+i/5.0)+0.3*sin(fTime*0.9+i/6.0);
	}
	glPopMatrix();
}
